unit BraidBasics;

interface

uses Basics;

{ A braid is an array of small integers, plus two integer parameters, }
{ L, which contains the length, and M, which contains (a bound for) }
{ the maximal generator occurring in the braid. }
{ A braid with negative length means "undefined". }
const BMaxLength = 2000;
const BMaxWidth = 50;
type braid = record
			V: packed array[1..BMaxLength] of -BMaxWidth..BMaxWidth;
			L, W: integer;
		end;
{ A list is an array of strings; the first entry is always "?"; used }
{ to mean "undefined". }
const maxBraidList = 10000;
const maxLengthInBraidList = 15;
type list = record
		V: array[0..maxBraidList] of string[maxLengthInBraidList];
		L: integer;
			end;
var braidList: list;
	maxCodeLengthInBraidList, widthBound, lengthBound: integer;


{-----------------------------------------------}
{ BASIC FUNCTIONS }
{-----------------------------------------------}

procedure writeFix(w: string; l: integer);
function inf (i, j: integer): integer;
{ int(w) is the integer denoted in base 10 by the word w. }
function int (w: string): integer;
function sup (i, j: integer): integer;
function sup3 (i, j, k: integer): integer;

{-----------------------------------------------}
{ BRAID INPUT }
{-----------------------------------------------}

procedure BDef (var B: braid; w: string);
function BDelta (n: integer): braid;
procedure BEnter (var B: braid);
procedure BEnterPos (var B: braid);
procedure BEnter2 (var B1, B2: braid);
{ BGenerate(B, p, n) lets B be the p-th braid word of width n. }
{ BGenerated(p, n) is the same realized as a function. }
procedure BGenerate (var B: braid; p: longInt; n: integer);
function BGenerated (p: longInt; n: integer): braid;
{ BOne is the unit braid. }
function BOne: braid;
{ BPosGenerate(B, p, n) and BPosGenerated(p, n) are similar with positive braids. }
procedure BPosGenerate (var B: braid; p: longInt; n: integer);
function BPosGenerated (p: longInt; n: integer): braid;
{ BSigma(i) is sigma_i. }
function BSigma (i: integer): braid;
function BUndefined: braid;

function thePositiveWord (p: LongInt; n: integer): string;
function thePositiveBraid (p: LongInt; n: integer): braid;

{-----------------------------------------------}
{ BRAID OUTPUT }
{-----------------------------------------------}

function BWord (B: braid): string;
procedure BWrite (B: braid);
procedure BWriteLn (B: braid);
procedure BSpecialWriteLn (B: braid; p, q: integer);

procedure BPrint (B: braid);
procedure BPrintLn (B: braid);
{ BPrintStyle indicates how braids are represented on the screen and in}
{ the output files. The possibilities are 'A', '1' or 's' (for sigma). }
var BPrintStyle: char;
{ BSpecialPrintLn(B, p, q) emphasizes the letters between positions p and q. }
procedure BSpecialPrintLn (B: braid; p, q: integer);
var TeX: text;
{-----------------------------------------------}
{                 BRAID BASICS                  }
{-----------------------------------------------}

{ BConcat(B, BB) multiplies B by BB and calls the result B. }
procedure BConcat(var B: braid; BB: braid);
{ BCopy(B, BB) copies B into BB. }
procedure BCopy(B: braid; var BB: braid);
{ BEqual(B, BB) is true iff B and BB coincide. }
function BEqual(B, BB: braid): Boolean;
{ BExtract(B, p, q, BB) copies in BB the subword of B between the }
{ positions p and q. }
function BExtracted (B: braid; p, q: integer): braid;
procedure BExtract (B: braid; p, q: integer; var BB: braid);
{ BFlip(B) flips B, i.e., switches sigma_1 and sigma_n, sigma_2 }
{ and sigma_(n-1), etc... The "n" taken into account is B.W. }
procedure BFlip (var B: braid);
{ BFreereduce(B) replaces B by its F-reduction (F like free), ie, }
{ the word obtained from B by deleting all possible pairs 'Aa' or 'aA'. }
procedure BFreereduce (var B: braid);
{ BInsert(B, p, q, BB) inserts in B the braid BB between the }
{ positions p and q. The length of B changes accordingly if the }
{ length of BB is not q - p + 1. }
procedure BInsert (var B: braid; p, q: integer; BB: braid);
procedure BInv (var B: braid);
procedure BInvWord (var w: string);
function BInversed (B: braid): braid;
function BNegPrefix (B: braid): braid;
function BPositive (B: braid): boolean;
function BPosPrefix (B: braid): braid;
{ BProd(B1, B2) is the product of B1 and B2.}
function BProd (B1, B2: braid): braid;
{ BSplit(B, BB) splits B into two halfs, call the first one B, and }
{ the second one BB. }
procedure BSplit(var B, BB: braid);

{-----------------------------------------------}
{ BRAID REVERSING }
{-----------------------------------------------}

function BLeftDen (B: braid): braid;
function BLeftNum (B: braid): braid;
procedure BLeftReverseOne (var B: braid);
procedure BLeftReverse (var B: braid);
procedure BLeftRightReverse (var B: braid);
var BNbRevSteps: longInt;
var BReversed: boolean;
function BRightDen (B: braid): braid;
function BRightNum (B: braid): braid;
procedure BRightReverseOne (var B: braid);
procedure BRightReverse (var B: braid);
procedure BRightLeftReverse (var B: braid);

{-----------------------------------------------}
{ LATTICE OPERATIONS }
{-----------------------------------------------}

function BLeftDivides (B1, B2: braid): boolean;
function BLeftDivInf (B1, B2: braid): braid;
function BLeftDivSup (B1, B2: braid): braid;
function BLeftGcd (B1, B2: braid): braid;
function BRightLcm (B1, B2: braid): braid;

{ BLeftQuotient(BB, B) is the result of left-dividing B by BB; }
{ if B and BB are positive, then the result is a positive braid }
{ if and only if BB left-divides B. }
function BLeftQuotient(BB, B: braid): braid;
function BRightQuotient (B, BB: braid): braid;

{-----------------------------------------------}
{ GARSIDE NORMAL FORM }
{-----------------------------------------------}

{ BExtractDelta(B, n, i, BB) extracts the maximal power i of Delta_n }
{ that left-divides B, and calls the quotient BB.}
procedure BExtractDelta(B: braid; n: integer; var i: integer; var BB: braid);
function BLeftHead (B: braid): braid;
function BLeftNormalWord (B: braid; sep: string): string;
function BSymLeftNormalWord (B: braid; sep: string): string;
{ garNormal(w) is the word that is the Garside normal form of }
{ braid that is represented by w, with no separation. }
function garNormal(w: string): string;
function paddedGarNormal(w: string): string;
{ Introducing a specific type for the normal form may be convenient }
{ as one directly accesses the data; however, the current version is }
{ mixed as the positive part is a string, so accessing the head is }
{ not so easy. Not sure one should keep that. } 
type NF = record
			I: integer;
			W: integer;
			S: string;
		end;
		procedure NFWrite(X: NF);
function NFBraid(X: NF): braid;
function BLeftNF (B: braid): NF;

{-----------------------------------------------}
{           ALTERNATING NORMAL FORM             }
{-----------------------------------------------}

{ One obtains the alternating normal form (a word) by }
{ successively dividing on the right by generators sigma_i. }
{ The point is to know by which generator to divide next when }
{ the current sigma_i does not work. To this end, one uses a }
{ tree of addresses, starting from 000... which corresponds to }
{ sigma_1 and then using the parity of the digits in the address. }
{ The addresses for B_n are length n sequences of integers. }
{ If needed, the breadth is the second digit in the last address, }
{ plus 1. }

function BAltNormalWord(B: Braid): string;
function altNormal(w: string): string;
function paddedAltNormal(w: string): string;
function altCode3(w: string): string;
function paddedAltCode3(w: string): string;
function altDecode3(w: string): string;
function garDecode3(w: string): string;

implementation

{-----------------------------------------------}
{ BASIC FUNCTIONS }
{-----------------------------------------------}
	
procedure writeFix(w: string; l: integer);
var r: integer;
begin
	write(w);
	for r:= 1 to l - length(w) do
		write(' ');
end;		{ writeFix }

function sup (i, j: integer): integer;
begin
	if i < j then
		sup := j
	else
		sup := i;
end; { sup }

function sup3 (i, j, k: integer): integer;
begin
	if i < j then
		sup3 := sup(j, k)
	else
		sup3 := sup(i, k);
end; { sup3 }

function inf (i, j: integer): integer;
begin
	if i < j then
		inf := i
	else
		inf := j;
end; { inf }

{ int(w) is the integer denoted in base 10 by the word w. }
function int (w: string): integer;
var
	p, k: integer;
begin
	p := 0;
	for k := 1 to Length(w) do
		p := (p * 10) + Ord(w[k]) - 48;
	Int := p;
end; { int }

{-----------------------------------------------}
{ BRAID INPUT }
{-----------------------------------------------}

procedure BDef (var B: braid; w: string);
var
	k, x: integer;
begin
	B.L := 0;
	B.W := 1;
	for k := 1 to Length(w) do
		begin
			x := Ord(w[k]);
			if (64 < x) and (x <= 90) then
				begin
					B.L := B.L + 1;
					B.V[B.L] := 64 - x;
					B.W := sup(B.W, x - 63);
				end;
			if (96 < x) and (x <= 122) then
				begin
					B.L := B.L + 1;
					B.V[B.L] := x - 96;
					B.W := sup(B.W, x - 95);
				end;
		end;
end; { BDef }

function BDelta(n: integer): braid;
var
	B: braid;
begin
	case n of
		1: BDef(B, '');
		2: BDef(B, 'a');
		3: BDef(B, 'aba');
		4: BDef(B, 'abacba');
		5: BDef(B, 'abacbadcba');
		6: BDef(B, 'abacbadcbaedcba');
		7: BDef(B, 'abacbadcbaedcbafedcba');
		8: BDef(B, 'abacbadcbaedcbafedcbagfedcba');
		9: BDef(B, 'abacbadcbaedcbafedcbagfedcbahgfedcba');
		10: BDef(B, 'abacbadcbaedcbafedcbagfedcbahgfedcbaihgfedcba');
		11: BDef(B, 'abacbadcbaedcbafedcbagfedcbahgfedcbaihgfedcbajihgfedcba');
		otherwise BDef(B, 'abacbadcbaedcbafedcbagfedcbahgfedcbaihgfedcbajihgfedcbakjihgfedcba');
		end;
	BDelta := B;
end;	{ BDelta }

procedure BEnter (var B: braid);
var w: string[255];
begin
	writeLn('Enter a braid word, using "a" for "sigma_1" and "A" for "sigma_1^{-1}" etc.');
	write('    > ');
	readLn(w);
	BDef(B, w);
end;	// BEnter

procedure BEnterPos (var B: braid);
var w: string[255];
begin
	writeLn('Enter a positive braid word, using "a" for "sigma_1" etc.');
	write('    > ');
	readLn(w);
	BDef(B, w);
end;	// BEnterPos

procedure BEnter2 (var B1, B2: braid);
var
	w: string[255];
begin
	writeLn('Enter two braid words, using "a" for "sigma_1" and "A" for "sigma_1^{-1}" etc.');
	write('    > B1 = ');
	readLn(w);
	BDef(B1, w);
	write('    > B2 = ');
	readLn(w);
	BDef(B2, w);
end; { BEnter2 }

{ BGenerate(B, p, n) lets B be the p-th braid word of width n. }
{ BGenerated(p, n) is the same realized as a function. }
procedure BGenerate (var B: braid; p: longInt; n: integer);
var
	w: string;
	x, nn: Integer;
begin
	nn := n - 1;
	w := '';
	while p > 0 do
		begin
			x := p mod (2 * nn);
			if x = 0 then
				x := 2 * nn;
			p := (p - x) div (2 * nn);
			if x <= nn then
				w := concat(Chr(96 + x), w)
			else
				w := concat(Chr(64 + x - nn), w);
		end;
	BDef(B, w);
end; 	{ BGenerate }

function BGenerated (p: LongInt; n: Integer): Braid;
	var
		B: Braid;
begin
	BGenerate(B, p, n);
	BGenerated := B;
end; 	{ BGenerated }

{ BOne is the unit braid. }
function BOne: Braid;
begin
	BOne.W := 1;
	BOne.L := 0;
end; { BOne }

{ BPosGenerate(B, p, n) lets B be the p-th positive braid word of width n. }
{ BPosGenerated(p, n) is the same realized as a function. }
procedure BPosGenerate (var B: braid; p: longInt; n: integer);
var
	w: string;
	x, nn: Integer;
begin
	nn := n - 1;
	w := '';
	while p > 0 do
		begin
			x := p mod nn;
			if x = 0 then
				x := nn;
			p := (p - x) div nn;
			w := concat(Chr(96 + x), w);
		end;
	BDef(B, w);
end; 	{ BPosGenerate }

function BPosGenerated (p: LongInt; n: Integer): Braid;
	var
		B: Braid;
begin
	BPosGenerate(B, p, n);
	BPosGenerated := B;
end; 	{ BPosGenerated }

{ BSigma(i) is sigma_i. }
function BSigma (i: integer): braid;
begin
	BSigma.L := 1;
	BSigma.W := i + 1;
	BSigma.V[1] := i;
end; { BSigma }

function BUndefined: braid;
begin
	BUndefined.W := 0;
	BUndefined.L := 0;
end; { BUndefined }

function thePositiveWord (p: LongInt; n: integer): string;
var
	w: string;
begin
	w := '';
	if p > 0 then
		if p < n then
			w := Chr(96 + p)
		else
			w := Concat(thePositiveWord((p - 1) div (n - 1), n), Chr(97 + (p - 1) mod (n - 1)));
		thePositiveWord := w;
end;	{ thePositiveWord }

function thePositiveBraid (p: LongInt; n: integer): braid;
begin
	BDef(thePositiveBraid, thePositiveWord(p, n));
end;			{ thePositiveBraid }

{-----------------------------------------------}
{ BRAID OUTPUT }
{-----------------------------------------------}

{ BWrite(B) writes the braid B on the screen.}
{ BWriteLn(B) is similar with a CR at the end. }
{ BSpecialwriteLn(B, p, q) emphasizes the letters between }
{ positions p and q. }

function BWord (B: braid): string;
var
	r: integer;
	w: string;
begin
	w := '';
	if B.W <= 0 then
		w := '?'
	else if B.L = 0 then
		w := '';
	for r := 1 to B.L do
		if B.V[r] > 0 then
			w := concat(w, chr(B.V[r] + 96))
		else
			w := concat(w, chr(64 - B.V[r]));
	BWord := w;
end; { BWord }

procedure BWrite (B: braid);
var
	r: integer;
begin
	if B.W = 0 then
		write('??ined')
	else if B.L = 0 then
		write('1');
	for r := 1 to B.L do
		begin
			if B.V[r] > 0 then
				write(Chr(B.V[r] + 96))
			else
				write(Chr(64 - B.V[r]));
			if r mod 65 = 0 then
				begin
					writeLn;
					write('     ');
				end;
		end;
end; { BWrite }

procedure BWriteLn (B: braid);
begin
	BWrite(B);
	writeLn;
end; { BWriteLn }

procedure BSpecialWriteLn (B: braid; p, q: integer);
var
	r: integer;
begin
	if B.L = 0 then
		write('1');
	for r := 1 to B.L do
		begin
			if (r = p) or (r = q + 1) then
        write('.');
      if B.V[r] > 0 then
				write(Chr(B.V[r] + 96))
			else
				write(Chr(64 - B.V[r]));
			if r mod 80 = 0 then
				begin
					writeLn;
					write('     ');
				end;
		end;
	if B.L = q then
    write('.');
  writeLn;
end;	// BSpecialWriteLn

procedure BPrint (B: braid);
var
	r: integer;
begin
	case BPrintStyle of
		'A': 
			begin
				if B.L = 0 then
					write(TeX, '$\e$ ');
				write(TeX, ' \ni $\tt ');
				for r := 1 to B.L do
					begin
						if B.V[r] > 0 then
							write(TeX, Chr(B.V[r] + 96))
						else
							write(TeX, Chr(64 - B.V[r]));
						if r mod 65 = 0 then
							writeLn(TeX, '\hfill\break\indent ');
					end;
				writeLn(TeX, '$');
			end;
		'1': 
			begin
				if B.L = 0 then
					write(TeX, '$1$ ');
				write(TeX, ' \ni $');
				for r := 1 to B.L do
					begin
						if B.V[r] > 0 then
							write(TeX, ' \; ', B.V[r])
						else
							write(TeX, ' \;\bar', -B.V[r]);
						if r mod 20 = 0 then
							write(TeX, '\hfill \break \indent ');
					end;
				writeLn(TeX, '$');
			end;
		's': 
			begin
				if B.L = 0 then
					write(TeX, '$1$ ');
				write(TeX, ' \ni $');
				for r := 1 to B.L do
					begin
						if B.V[r] > 0 then
							write(TeX, ' \s_', B.V[r])
						else
							write(TeX, ' \s_', -B.V[r], '^{-1}');
						if r mod 20 = 0 then
							write(TeX, '\hfill \break \indent ');
					end;
				writeLn(TeX, '$ ');
			end;
	end;
end; { BPrint }

procedure BPrintLn (B: braid);
begin
	BPrint(B);
	writeLn(TeX);
	writeLn(TeX);
end; { BPrintLn }

procedure BSpecialPrintLn (B: braid; p, q: integer);
var
	r: integer;
begin
	write(TeX, ' \ni $\tt ');
	for r := 1 to B.L do
		begin
			if r = p then
				begin
					write(TeX, '\underline{ ');
					write('  ');
				end;
			if B.V[r] > 0 then
				begin
					write(TeX, Chr(B.V[r] + 96));
					write(Chr(B.V[r] + 96));
				end
			else
				begin
					write(TeX, Chr(64 - B.V[r]));
					write(Chr(64 - B.V[r]));
				end;
			if r = q then
				begin
					write(TeX, '} ');
					write('  ');
				end;
			if r mod 70 = 0 then
				write(TeX, '\hfill\break\indent ');
		end;
	writeLn(TeX, '$');
	writeLn;
	writeLn(TeX);
	writeLn(TeX);
end; { BSpecialPrintLn }

{-----------------------------------------------}
{ BRAID BASICS }
{-----------------------------------------------}

{ BConcat(B, BB) multiplies B by BB and calls the result B. }
procedure BConcat (var B: braid; BB: braid);
var
	r: integer;
begin
	B.W := sup(B.W, BB.W);
	for r := 1 to BB.L do
		B.V[B.L + r] := BB.V[r];
	B.L := B.L + BB.L;
end; { BConcat }

{ BCopy(B, BB) copies B into BB. }
procedure BCopy (B: braid; var BB: braid);
	var
		r: integer;
begin
	for r := 1 to B.L do
		BB.V[r] := B.V[r];
	BB.L := B.L;
	BB.W := B.W;
end;

{ BEqual(B, BB) is true iff B and BB coincide. }
function BEqual (B, BB: braid): Boolean;
var
	r: integer;
begin
	if B.L <> BB.L then
		begin
			BEqual := False;
			exit(BEqual);
		end;
	r := 0;
	repeat
		r := r + 1;
	until (r > B.L) or (B.V[r] <> BB.V[r]);
	BEqual := (r > B.L);
end; { BEqual }

procedure BExtract (B: braid; p, q: integer; var BB: braid);
var
	r, s: integer;
begin
	s := 0;
	for r := p to q do
		if (r >= 1) and (r <= B.L) then
			begin
				s := s + 1;
				BB.V[s] := B.V[r];
			end;
	BB.L := s;
	BB.W := B.W;
end; { BExtract }

{ BExtract(B, p, q, BB) copies in BB the subword of B between the }
{ positions p and q. }
function BExtracted (B: braid; p, q: integer): braid;
var
	r, s: integer;
begin
	s := 0;
	for r := p to q do
		if (r >= 1) and (r <= B.L) then
			begin
				s := s + 1;
				BExtracted.V[s] := B.V[r];
			end;
	BExtracted.L := s;
	BExtracted.W := B.W;
end; { BExtracted }

{ BFlip(B) flips B, i.e., switches sigma_1 and sigma_n, sigma_2 }
{ and sigma_(n-1), etc... The "n" taken into account is B.W. }
procedure BFlip (var B: braid);
var
	r: integer;
begin
	for r := 1 to B.L do
		if B.V[r] > 0 then
			B.V[r] := B.W - B.V[r]
		else
			B.V[r] := -B.W - B.V[r];
end; { BFlip }

{ BFreereduce(B) replaces B by its F-reduction (F like free), ie, }
{ the word obtained from B by deleting all possible pairs 'Aa' or 'aA'. }
procedure BFreereduce (var B: braid);
var
	p, r: integer;
begin
	p := 1;
	while p < B.L do
		begin
			if B.V[p] + B.V[p + 1] = 0 then
				begin
					B.L := B.L - 2;
					for r := p to B.L do
						B.V[r] := B.V[r + 2];
					p := sup(p - 1, 1);
				end
			else
				p := p + 1;
		end;
end; { BFreereduce }

{ BInsert(B, p, q, BB) inserts in B the braid BB between the positions p and q. }
{ The length of B changes accordingly if the length of BB is not q - p + 1. }
procedure BInsert (var B: Braid; p, q: Integer; BB: Braid);
var
	m, r: Integer;
begin
	m := BB.L - q + p - 1;
	if m > 0 then
		for r := B.L downto q + 1 do
			B.V[r + m] := B.V[r];
	if m < 0 then
		for r := q + 1 to B.L do
			B.V[r + m] := B.V[r];
	for r := 1 to BB.L do
		B.V[p + r - 1] := BB.V[r];
	B.L := B.L + m;
	B.W := sup(B.W, BB.W);
end; { BInsert }

{ BInv(B) inverses B. }
procedure BInv (var B: braid);
var
	r, x: integer;
begin
	for r := 1 to B.L div 2 do
		begin
			x := B.V[r];
			B.V[r] := -B.V[B.L - r + 1];
			B.V[B.L - r + 1] := -x;
		end;
	if odd(B.L) then
		B.V[B.L div 2 + 1] := -B.V[B.L div 2 + 1];
end; { BInv }

function BInversed (B: braid): braid;
var
	r, x: integer;
begin
	for r := 1 to B.L div 2 do
		begin
			x := B.V[r];
			BInversed.V[r] := -B.V[B.L - r + 1];
			BInversed.V[B.L - r + 1] := -x;
		end;
	if odd(B.L) then
		BInversed.V[B.L div 2 + 1] := -B.V[B.L div 2 + 1];
	BInversed.L := B.L;
	BInversed.W := B.W;
end; { BInversed }

procedure BInvWord (var w: string);
var
	p: integer;
	ww: string;
begin
	ww := '';
	for p := 1 to length(w) do
		begin
			if (ord(w[p]) > 64) and (ord(w[p]) < 96) then
				ww := concat(chr(ord(w[p])+32), ww);
			if (ord(w[p]) > 96) and (ord(w[p]) < 128) then
				ww := concat(chr(ord(w[p])-32), ww);
			if w[p] = '.' then
				ww := concat('.', ww);
		end;
	w := ww;
end; { BInvWord }

{ BNegPrefix(B) is the largest positive prefix of B. }
function BNegPrefix (B: braid): braid;
var
	p: integer;
begin
	p := 0;
	repeat
		p:= p+1;
		if B.V[p] < 0 then BNegPrefix.V[p] := B.V[p];
	until (p > B.L) or (B.V[p] > 0);
	BNegPrefix.L := p-1;
	BNegPrefix.W := B.W;
end; { BNegPrefix }

{ BPositive(B) is TRUE if B is a positive braid word. }
function BPositive (B: braid): boolean;
var
	p: integer;
begin
	p := 0;
	repeat p := p+1 until (p > B.L) or (B.V[p] < 0);
	BPositive := (p > B.L);
end; { BPositive }

{ BPosPrefix(B) is the largest positive prefix of B. }
function BPosPrefix (B: braid): braid;
var
	p: integer;
begin
	p := 0;
	repeat
		p:= p+1;
		if B.V[p] > 0 then BPosPrefix.V[p] := B.V[p];
	until (p > B.L) or (B.V[p] < 0);
	BPosPrefix.L := p-1;
	BPosPrefix.W := B.W;
end; { BPosPrefix }

{ BProd(B1, B2) is the product of B1 and B2.}
function BProd (B1, B2: Braid): Braid;
var
	BB: Braid;
	r, s: Integer;
begin
	BB.W := sup(B1.W, B2.W);
	s := 0;
	if B1.L > 0 then
		for r := 1 to B1.L do
			begin
				s := s + 1;
				BB.V[s] := B1.V[r];
			end;
	if B2.L > 0 then
		for r := 1 to B2.L do
			begin
				s := s + 1;
				BB.V[s] := B2.V[r];
			end;
	BB.L := s;
	BProd := BB;
end;	{ BProd }

{ BSplit(B, BB) splits B into two halfs, call the first one B, and }
{ the second one BB. }
procedure BSplit (var B, BB: braid);
var
	r, l: integer;
begin
	l := B.L div 2;
	for r := 1 to B.L - l do
		BB.V[r] := B.V[r + l];
	BB.L := B.L - l;
	BB.W := B.W;
	B.L := l;
end;	{ BSplit }

{-----------------------------------------------}
{ BRAID REVERSING }
{-----------------------------------------------}
	
function BLeftDen (B: braid): braid;
var
	r: integer;
begin
	BLeftReverse(B);
	BInv(B);
	r:= 0;
	repeat
		r:= r+1;
	until (r > B.L) or (B.V[r] > 0);
	BLeftDen:= BExtracted(B, r, B.L);
end; { BLeftDen }

function BLeftNum (B: braid): braid;
var
	r: integer;
begin
	BLeftReverse(B);
	r:= 0;
	repeat
		r:= r+1;
	until (r > B.L) or (B.V[r] > 0);
	BLeftNum:= BExtracted(B, r, B.L);
end; { BLeftNum }

{ BLeftReverseOne performs one step of left-Reverseersing. }
procedure BLeftReverseOne(var B: braid);
	var
		i, k, t: integer;
begin
	i := 0;
	repeat
		i := i + 1
	until ((B.V[i] > 0) and (B.V[i + 1] < 0)) or (i = B.L);
	if i < B.L then
		begin
			t := B.V[i] + B.V[i + 1];
			if (t < -1) or (t > 1) then
				begin
					B.V[i] := B.V[i + 1];
					B.V[i + 1] := t - B.V[i + 1];
				end
			else if t = 0 then
				begin
					for k := i + 2 to B.L do
						B.V[k - 2] := B.V[k];
					B.L := B.L - 2;
				end
			else
				begin
					for k := B.L downto i + 2 do
						B.V[k + 2] := B.V[k];
					B.V[i + 3] := B.V[i];
					B.V[i + 2] := -B.V[i + 1];
					B.V[i + 1] := -B.V[i + 3];
					B.V[i] := -B.V[i + 2];
					B.L := B.L + 2;
				end;
		end
	else
		BReversed := True;
end; { BLeftReverseOne }

procedure BLeftReverse(var B: braid);
begin
	BReversed := FALSE;
	repeat
		BLeftReverseOne(B);
	until BReversed;
end; { BLeftReverse }

procedure BLeftRightReverse(var B: braid);
begin
	BLeftReverse(B);
	BRightReverse(B);
end; { BLeftRightReverse }

function BRightDen (B: braid): braid;
var
	r: integer;
begin
	BRightReverse(B);
	BInv(B);
	r:= 0;
	repeat
		r:= r+1;
	until (r > B.L) or (B.V[r] < 0);
	BRightDen:= BExtracted(B, 1, r-1);
end; { BRightDen }

function BRightNum (B: braid): braid;
var
	r: integer;
begin
	BRightReverse(B);
	r:= 0;
	repeat
		r:= r+1;
	until (r > B.L) or (B.V[r] < 0);
	BRightNum:= BExtracted(B, 1, r-1);
end; { BRightNum }

procedure BRightLeftReverse(var B: braid);
begin
	BRightReverse(B);
	BLeftReverse(B);
end; { BRightLeftReverse }

{ BRightReverseOne performs one step of left-Reverseersing. }
procedure BRightReverseOne(var B: braid);
var
	i, k, t: integer;
begin
	i := 0;
	repeat
		i := i + 1
	until ((B.V[i] < 0) and (B.V[i + 1] > 0)) or (i = B.L);
	if i < B.L then
		begin
			t := B.V[i] + B.V[i + 1];
			if (t < -1) or (t > 1) then
				begin
					B.V[i] := B.V[i + 1];
					B.V[i + 1] := t - B.V[i + 1];
				end
			else if t = 0 then
				begin
					for k := i + 2 to B.L do
						B.V[k - 2] := B.V[k];
					B.L := B.L - 2;
				end
			else
				begin
					for k := B.L downto i + 2 do
						B.V[k + 2] := B.V[k];
					B.V[i + 3] := B.V[i];
					B.V[i + 2] := -B.V[i + 1];
					B.V[i + 1] := -B.V[i + 3];
					B.V[i] := -B.V[i + 2];
					B.L := B.L + 2;
				end;
		end
	else
		BReversed := TRUE;
end; { BRightReverseOne }

procedure BRightReverse(var B: braid);
begin
	BReversed := FALSE;
	repeat
		BRightReverseOne(B);
	until BReversed;
end; { BRightReverse }

{-----------------------------------------------}
{ LATTICE OPERATIONS }
{-----------------------------------------------}

function BLeftDivides (B1, B2: braid): boolean;
begin
	BInv(B1);
	BConcat(B1, B2);
	BLeftRightReverse(B1);
	BLeftDivides := BPositive(B1);
end; { BLeftDivides}

function BLeftDivInf (B1, B2: braid): braid;
var B: braid;
begin
	B := BProd(BInversed(B1), B2);
	BRightReverse(B);
	BLeftReverse(B);
	B := BProd(B1, BNegPrefix(B));
	BLeftReverse(B);
	BLeftDivInf := B;
end; { BLeftDivInf }

function BLeftDivSup (B1, B2: braid): braid;
var B: braid;
begin
	B := BProd(BInversed(B1), B2);
	BLeftReverse(B);
	BRightReverse(B);
	B := BProd(B1, BPosPrefix(B));
	BRightReverse(B);
	BLeftDivSup := B;
end; { BLeftDivSup }

function BLeftGcd (B1, B2: braid): braid;
var B: braid;
begin
	if BPositive(B1) and BPositive(B2) then
		begin
			B := BProd(BInversed(B1), B2);
			BRightLeftReverse(B);
			B := BProd(B1, BNegPrefix(B));
			BLeftReverse(B);
			BLeftGcd := B;
		end
	else BLeftGcd := BUndefined;
end; { BLeftGcd }

function BRightLcm (B1, B2: braid): braid;
var B: braid;
begin
	if BPositive(B1) and BPositive(B2) then
		begin
			B := BProd(BInversed(B1), B2);
			BRightReverse(B);
			BRightLcm := BProd(B1, BPosPrefix(B));
		end
	else BRightLcm := BUndefined;
end; { BRightLcm }

{ BLeftQuotient(BB, B) is the result of left-dividing B by BB; }
{ if B and BB are positive, then the result is a positive braid }
{ if and only if BB left-divides B. }

function BLeftQuotient(BB, B: braid): braid;
begin
	B:= BProd(BInversed(BB), B);
	BRightReverse(B);
	BLeftQuotient:= B;
end;		{ BLeftQuotient }

function BRightQuotient (B, BB: braid): braid;
begin
	B := BProd(B, BInversed(BB));
	BLeftReverse(B);
	BRightQuotient := B;
end; { BRightQuotient }

{-----------------------------------------------}
{              GARSIDE NORMAL FORM              }
{-----------------------------------------------}

{ BExtractDelta(B, i, BB) extracts the maximal power i of Delta }
{ that left-divides B, and calls the quotient BB.}

procedure BExtractDelta(B: braid; n: integer; var i: integer; var BB: braid);
var
	hope: boolean;
begin
	i:= 0;
	BB:= B;
	hope:= TRUE;
	if BPositive(B) and (B.W > 1) then
		while hope do
			begin
				BB:= BLeftQuotient(BDelta(n), B);
				if BPositive(BB) then
					begin
						i:= i+1;
						B:= BB;
					end
				else
					begin
						hope:= FALSE;
						BB:= B;
					end;
			end;
end;		{ BExtractDelta }

function BLeftHead (B: braid): braid;
begin
	if BPositive(B) then
		BLeftHead := BLeftGcd(B, BDelta(B.W))
	else BLeftHead := BUndefined;
end;	{ BLeftHead }

function BLeftNormalWord (B: braid; sep: string): string;
var
	w: string;
	BB: braid;
	i, j: integer;
begin
	if BPositive(B) then
		begin
			BExtractDelta(B, 3, i, B);
			w:= '';
			for j:= 1 to i do
				w:= concat(w, 'D');
			w := concat(w, sep);
			repeat
				BB := BLeftHead(B);
				w := concat(w, sep, BWord(BB));
				B:= BLeftQuotient(BB, B);
			until B.L = 0;
			BLeftNormalWord := w;
		end
	else BLeftNormalWord := '??ined';
end;	{ BLeftNormalWord }

function BSymLeftNormalWord (B: braid; sep: string): string;
var
	w1, w2, w: string;
	B1, B2: braid;
begin
	BRightLeftReverse(B);
	B1 := BLeftDen(B);
	B2 := BLeftNum(B);
	w1 := BLeftNormalWord(B1, sep);
	BInvWord(w1);
	w2 := BLeftNormalWord(B2, sep);
	if length(w1) = 0 then
		w := w2
	else if length(w2) = 0 then
		w := w1
	else 
		w := concat(w1, sep, w2);
	BSymLeftNormalWord := w;
end; { BSymLeftNormalWord }

procedure NFWrite(X: NF);
begin
	write('D_', X.W, '^', X.I, '.', X.S);
end;		{ NFWrite }

function BLeftNF (B: braid): NF;
var
	w: string;
	BB: braid;
	i: integer;
begin
	if BPositive(B) then
		begin
			BLeftNF.W:= B.W;
			BExtractDelta(B, B.W, i, B);
			BLeftNF.I:= i;
			w := '';
			repeat
				BB := BLeftHead(B);
				if length(w) = 0 then
					w := BWord(BB)
				else
					w := concat(w, '.', BWord(BB));
				B:= BLeftQuotient(BB, B);
			until B.L = 0;
			BLeftNF.S := w;
		end
	else BLeftNF.S := '??ined';
end; { BLeftNF }

function NFEqual(X, Y: NF): boolean;
begin
	NFEqual := (X.I = Y.I) and (X.W = Y.W) and (X.S = Y.S);
end;		{ NFEqual }

function NFBraid(X: NF): braid;
var
	B: Braid;
	p: integer;
begin
	BDef(B, X.S);
	for p:= 1 to X.I do
		B:= BProd(BDelta(X.W), B);
	NFBraid:= B;
end;		{ NFBraid }

{ garNormal(w) is the word that is the Garside normal form of }
{ braid that is represented by w, with no separation. }
function garNormal(w: string): string;
var B: braid;
begin
	if w = '?' then
		begin
			garNormal:= '?';
			exit;
		end;
	BDef(B, w);
	garNormal:= BLeftNormalWord(B, '');
end;		{ garNormal }

function paddedGarNormal(w: string): string;
var u: string;
	e: integer;
begin
	u:= garNormal(w);
	for e:= 1 to (lengthBound - length(u)) do
		u:= concat(u, ' ');
	paddedGarNormal:= u;
end;		{ paddedGarNormalB }


{-----------------------------------------------}
{           ALTERNATING NORMAL FORM             }
{-----------------------------------------------}

{ One obtains the alternating normal form (a word) by }
{ successively dividing on the right by generators sigma_i. }
{ The point is to know by which generator to divide next when }
{ the current sigma_i does not work. To this end, one uses a }
{ tree of addresses, starting from 000... which corresponds to }
{ sigma_1 and then using the parity of the digits in the address. }
{ The addresses for B_n are length n sequences of integers. }
{ If needed, the breadth is the second digit in the last address, }
{ plus 1. }

type
	address = record
		V: array[1..BMaxWidth] of integer;
		L: integer;
		end;

procedure enterAdd(var a: address);
var 
	w: string;
	p: integer;
begin
	writeLn('Enter address');
	readLn(w);
	for p:= 1 to length(w) do
		a.V[p]:= Ord(w[p])-96;
	a.L:= length(w);
end;		{ enterAdd }

procedure writeAdd(a: address);
var 
	p: integer;
begin
	for p:= 1 to a.L do
		write(a.V[p]);
end;		{ writeAdd }

function firstAdd(n: integer): address;
var
	p: integer;
begin
	for p:= 1 to n do
		firstAdd.V[p]:= 0;
	firstAdd.L:= n;
end;		{ firstAdd }

function nextAdd(var a: address; p: integer): address;
var
	q: integer;
begin
	nextAdd.L:= a.L;
	for q:= 1 to p-1 do
		nextAdd.V[q]:= a.V[q];
	nextAdd.V[p]:= a.V[p]+1;
	for q:= p+1 to a.L do
		nextAdd.V[q]:= 0;
end;		{ nextAdd }

function generator(a: address): integer;
var
	p, i: integer;
begin
	i:= 1;
	for p:= 1 to a.L - 1 do
		if odd(a.V[a.L - p + 1]) then
			i:= p - i + 1;
	generator:= i;
end;		{ generator }

function BAltNormalWord(B: Braid): string;
var
	w: string;
	a: address;
	n, i, p: integer;
	BB: Braid;
begin
	n:= B.W;
	w:= '';
	a:= firstAdd(n);
	p:= n;
	repeat
		i:= generator(a);
		BB:= BProd(B, BInversed(BSigma(i)));
		BLeftReverse(BB);
		if BB.L < B.L then
			begin
				w:= concat(chr(96+i), w);
				B:= BB;
				p:= n;
			end
		else
			p:= p-1;
		a:= nextAdd(a, p);
	until B.L = 0;
	BAltNormalWord:= w;
end;		{ BAltNormalWord }

function altNormal(w: string): string;
var B: braid;
begin
	if w = '?' then
		begin
			altNormal:= '?';
			exit;
		end;
	BDef(B, w);
	altNormal:= BAltNormalWord(B);
end;		{ altNormal }

function paddedAltNormal(w: string): string;
var u: string;
	e: integer;
begin
	u:= altNormal(w);
	for e:= 1 to (lengthBound - length(u)) do
		u:= concat(' ', u);
	paddedAltNormal:= u;
end;		{ paddedAltNormalB }

function hexaCode(e: integer): char;
begin
	if e <= 9 then
		hexaCode:= chr(48+e)
	else
		hexaCode:= chr(55+e);
end;		{ hexaCode }

function hexaDecode(c: char): integer;
begin
	if c < 'A' then
		hexaDecode:= ord(c)-48
	else
		hexaDecode:= ord(c)-54;
end;		{ hexaDecode }

function code3(w: string): string;
var u: string;
	c: char;
	e: integer;
begin
	if w = '?' then
		begin
			code3:= '?';
			exit;
		end;
	u:= '';
	c:= 'a';
	while length(w) > 0 do
		begin
			e:= 0;
			while w[length(w)] = c do
				begin
					w:= copy(w, 1, length(w)-1);
					e:= e+1;
				end;
			u:= concat(hexaCode(e), u);
			if c = 'a' then c:= 'b' else c:= 'a';
		end;
	code3:= u;
end;		{ code3 }
		
function altCode3(w: string): string;
begin
	altCode3:= code3(altNormal(w));
end;		{ altCode3 }

function paddedAltCode3(w: string): string;
var u: string;
	e: integer;
begin
	u:= altCode3(w);
	for e:= 1 to (maxCodeLengthInBraidList - length(u)) do
		u:= concat(' ', u);
		paddedAltCode3:= u;
end;		{ altCode3 }

function altDecode3(w: string): string;
var u: string;
	p, r, e: integer;
	c: char;
begin
	p:= pos(',', w);
	if p > 0 then
		altDecode3:= concat(altDecode3(copy(w, 1, p-1)), ',', altDecode3(copy(w, p+1, 255)))
	else
		begin
			u:= '';
			c:= 'a';
			for r:= length(w) downto 1 do
				begin
					for e:= 1 to hexaDecode(w[r]) do u:= concat(c, u);
					if c = 'a' then c:= 'b' else c:= 'a';
				end;
			altDecode3:= u;
		end;
end;		{ altDecode3 }
		
function garDecode3(w: string): string;
var u: string;
	p, r, e: integer;
	c: char;
begin
	if w = '?' then
		begin
			garDecode3:= '?';
			exit;
		end;
	p:= pos(',', w);
	if p > 0 then
		garDecode3:= concat(garDecode3(copy(w, 1, p-1)), ',', garDecode3(copy(w, p+1, 255)))
	else
		begin
			u:= '';
			c:= 'a';
			for r:= length(w) downto 1 do
				begin
					for e:= 1 to hexaDecode(w[r]) do u:= concat(c, u);
					if c = 'a' then c:= 'b' else c:= 'a';
				end;
			garDecode3:= garNormal(u);
		end;
end;		{ garDecode3 }
		

end.
