unit BraidConjugacy;

interface

uses Basics, BraidBasics, HandleReduction;

var SSSList: array[1..7000] of string;
var mu3, mu3SSS, mu3Conj, mu3SSSConj: array[1..7000] of integer;

  {-----------------------------------------------}
  {            LISTS OF (BRAID) WORDS             }
  {-----------------------------------------------}

  var found: boolean;
  function emptyList: list;
  procedure initList(w: string; var L: list);
  procedure insertList(w: string; var L: list);
  procedure writeList(L: list);
  procedure copyList(L: list; var LL: list);
  procedure buildBraidList;
  { rank(B) is the position where B occurs in braidList, if any, and -1 otherwise. }
  function rank(B: braid): integer;

{-----------------------------------------------}
{                      SSS3                     }
{-----------------------------------------------}

function NFCycled(X: NF): NF;
function cycled(w: string): string;
function SSS3AsList(w: string): list;
function SSS3AsString(w: string): string;
function SSS3AsAltCodedString(w: string): string;

function BConjugator3(p, q: integer): integer;

procedure buildSSS3List;
procedure buildMu3;

implementation

{-----------------------------------------------}
{            LISTS OF (BRAID) WORDS             }
{-----------------------------------------------}

function emptyList: list;
begin
	emptyList.L:= 0;
	emptyList.V[0]:= '?';
end;		{ emptyList }

procedure writeList(L: list);
var
	p: integer;
begin
	write('(');
	if L.L > 0 then
		begin
			for p:= 1 to L.L-1 do
				write(L.V[p], '; ');
			write(L.V[L.L]);
		end;
	write(')');
end;		{ writeList }

procedure splitList(L: list; var L1, L2, L3: list);
var
	m, p: integer;
begin
	m:= (L.L+1) div 2;
	L1.L:= m-1;
	for p:= 1 to L1.L do
		L1.V[p]:= L.V[p];
	L2.L:= 1;
	L2.V[1]:= L.V[m];
	L3.L:= L.L - m;
	for p:= 1 to L3.L do
		L3.V[p]:= L.V[p + m];
end;		{ splitList }

procedure concatList(L1, L2, L3: list; var L: list);
var
	p: integer;
begin
	L.L:= L1.L + 1 + L3.L;
	for p:= 1 to L1.L do
		L.V[p]:= L1.V[p];
	L.V[L1.L + 1]:= L2.V[1];
	for p:= 1 to L3.L do
		L.V[L1.L + 1 + p]:= L3.V[p];
end;		{ concatList }

procedure copyList(L: list; var LL: list);
var p: integer;
begin
	LL.L:= L.L;
	for p:= -1 to L.L do
		LL.V[p]:= L.V[p];
end;		{ copyList }

procedure initList(w: string; var L: list);
begin
	L.L:= 1;
	L.V[0]:= '?';
	L.V[1]:= w;
end;		{ initList }

procedure insertList(w: string; var L: list);
var
	L1, L2, L3: list;
	B, BB: braid;
begin
	found:= FALSE;
	if L.L = 0 then
		begin
			L.L:= 1;
			L.V[1]:= w;
		end
	else
		begin
			splitList(L, L1, L2, L3);
			BDef(B, L2.V[1]);
			BDef(BB, w);
			case BSign(BProd(BInversed(B), BB)) of
				'-': begin
						insertList(w, L1);
						concatList(L1, L2, L3, L);
					end;
				'+': begin
						insertList(w, L3);
						concatList(L1, L2, L3, L);
					end;
				'1': found:= TRUE;
				end;
		end;
end;		{ insertList }

procedure buildBraidList;
var p: longint;
	w: string;
begin
	writeLn('Constructing the braid table.');
	braidList:= emptyList;
	p:= 0;
	w:= '';
	repeat
		insertList(w, braidList);
		if length(altCode3(w)) > maxCodeLengthInBraidList then
			maxCodeLengthInBraidList:= length(altCode3(w));
		p:= p+1;
		w:= BAltNormalWord(thePositiveBraid(p, widthBound));
	until length(w) > lengthBound;
	write('Construction completed: there exist ', braidList.L);
	writeLn(' positive braids of length at most ', lengthBound, ' in B_3.');
end;		{ buildBraidList }

{ rank(B) is the position where B occurs in braidList, if any, and -1 otherwise. }
function rank(B: braid): integer;
var p: integer;
	BB: braid;
	found: boolean;
begin
	p:= 0;
	repeat
		p:= p+1;
		BDef(BB, braidList.V[p]);
		found:= BEquiv(B, BB);
	until found or (p = braidList.L);
	if found then
		rank:= p
	else
		rank:= -1;
end;		{ rank }

	
{-----------------------------------------------}
{                      SSS3                     }
{-----------------------------------------------}

function flip(u: string; i, n: integer): string;
var p: integer;
begin
	if odd(i) then
		for p:= 1 to length(u) do
			u[p]:= chr(192 + n - ord(u[p]));
	flip:= u;
end;		{ flip }

function NFCycled(X: NF): NF;
var
	p: integer;
	u: string;
	B: braid;
begin
	if pos('.', X.S) = 0 then
		NFCycled:= X
	else
		begin
			p:= pos('.', X.S);
			u:= copy(X.S, 1, p-1);
			u:= flip(u, X.I, X.W);
			X.S:= concat(copy(X.S, p+1, 255), '.', u);
			NFCycled:= X;
		end;
	B:= NFBraid(X);
	NFCycled:= BLeftNF(B);
end;		{ NFCycled }

function cycled(w: string): string;
var
	B: braid;
	X: NF;
begin
	BDef(B, w);
	X:= BLeftNF(B);
	X:= NFCycled(X);
	B:= NFBraid(X);
	cycled:= BAltNormalWord(B);
end;		{ cycled }

function SSS3AsList(w: string): list;
var
	L, LL: list;
	r: integer;
	v: string;
begin
	initList(w, L);
	repeat
		w:= cycled(w);
		insertList(w, L);
	until found;
	initList(w, L);
{ At this point one found a word w that represents a braid whose cycling orbit is periodic. }
{ So, starting from the latter, one constructs the periodic part of the cycling orbit of }
{ the initial braid. }
	repeat
		w:= cycled(w);
		insertList(w, L);
	until found;
{ Now it remain to add the Delta-conjugates of the entries of the list. }
	copyList(L, LL);
	for r:= 1 to LL.L do
		begin
			v:= altNormal(flip(LL.V[r], 1, 3));
			insertList(v, L);
		end;
	SSS3AsList:= L;
end;		{ SSS3AsList }

function SSS3AsString(w: string): string;
var 
	L: list;
	u: string;
	r: integer;
begin
	L:= SSS3AsList(w);
	u:= L.V[1];
	for r:= 2 to L.L do
		u:= concat(u, ';', L.V[r]);
	SSS3AsString:= u;
end;		{ SSS3AsString }

function SSS3AsAltCodedString(w: string): string;
var 
	L: list;
	u: string;
	r: integer;
begin
	L:= SSS3AsList(w);
	u:= concat(altCode3(L.V[1]));
	for r:= 2 to L.L do
		u:= concat(u, ',', altCode3(L.V[r]));
	SSS3AsAltCodedString:= u;
end;		{ SSS3AsAltCodedString }

function rankFirstEntry(w: string): integer;
var p: integer;
	B: braid;
begin
	p:= pos(',', w);
	if p > 0 then
		w:= copy(w, 1, p-1);
	w:= altDecode3(w);
	BDef(B, w);
	rankFirstEntry:= rank(B);
end;		{ rankFirstEntry }

procedure buildSSS3List;
var p: integer;
begin
	writeLn('Constructing the table of SSS.');
	for p:= 1 to braidList.L-1 do
		begin
			SSSList[p]:= SSS3AsAltCodedString(braidList.V[p]);
			Mu3SSS[p]:= rankFirstEntry(SSSList[p]);
			writeLn(p, ' braids have been treated.');
		end;
		
end;		{ buildSSS3List }

function BConjugator3(p, q: integer): integer;
var r: integer;
	B1, B2, B, BB: braid;
	found: boolean;
begin
	found:= FALSE;
	BDef(B1, braidList.V[p]);
	BDef(B2, braidList.V[q]);
	r:= 1;
	repeat
		BDef(B, braidList.V[r]);
		BB:= BProd(B2, B);
		BB:= BProd(BInversed(B), BB);
		found:= BEquiv(B1, BB);
		r:= r+1;
	until found or (r > braidList.L);
	if found then
		BConjugator3:= r-1
	else
		BConjugator3:= 0;
	end;		{ BConjugator3 }

procedure buildMu3;
var p, q: integer;
begin
	for p:= 1 to braidList.L do
		begin
			q:= 0;
			repeat
				q:= q+1
			until SSSList[q] = SSSList[p];
			mu3[p]:= q;
			mu3Conj[p]:= BConjugator3(p, q);
			mu3SSSConj[p]:= BConjugator3(p, mu3SSS[p]);
		end;
end;		{ buildMu3 }

end.
