program Braids;

uses Basics, BraidBasics, BraidConjugacy, HandleReduction;

var
	menu: char;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Handle reduction
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure reduceHandles;
var B: braid;
  k: integer;
begin
  hRule;
	writeLn('This procedure displays handle reduction w.r.t. the leftmost strategy');
  writeLn('i.e., one always reduces the leftmost handle.');
  BEnter(B);
	BStepByStepHReduce(B);
  hRule;
end;	// reduceHandles


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Comparison
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure compare;
var
	B1, B2: braid;
begin
  hRule;
  writeLn('This procedure compares two braids with respect to the chosen braid ordering.');
	BEnter2(B1, B2);
	BInv(B1);
	BConcat(B1, B2);
	BReduce(B1);
	if BVisibleSign(B1) = '1' then
		writeLn('We have B1 = B2.')
	else if BVisibleSign(B1) = '+' then
		begin
			write('We have B1 < B2, since B1^{-1}.B2 = ');
			BWrite(B1);
			writeLn(',');
			writeLn('and the quotient is larger than 1 in ', BOrderName, '.');
		end
	else
		begin
			write('We have B1 > B2, namely B2 = B1 * ');
			BWrite(B1);
			writeLn(',');
			writeLn('and the quotient is smaller than 1 in ', BOrderName, '.');
		end;
  hRule;
end; // compare

procedure equivalent;
var
	B, BB: braid;
begin
	BEnter(B);
	BEnter(BB);
	if BEquiv(B, BB) then
		writeLn('These braid words are equivalent.')
	else
		writeLn('These braid words are not equivalent.');
end; {equivalent}

procedure writeAlt;
var w: string;
	B: braid;
begin
	BEnterPos(B);
  w:= BAltNormalWord(B);
	write('The alternating normal form is ', w);
	//if B.W <= 3 then
	//	write(' with code ', altCode3(w));
	writeLn;
end;		{ writeAlt }

procedure symmetric;
var
	B: braid;
begin
	BEnter(B);
	writeLn('The symmetric normal decomposition is');
	writeLn('  ', BSymLeftNormalWord(B, '.'));
end; { symmetric }

procedure buildSSS3;
var p: integer;
	w, u: string;
begin
	widthBound:= 3;
	write('Until which length? ');
	readLn(lengthBound);
	buildBraidList;
	buildSSS3List;
	buildMu3;
	writeLn('Below is the enumeration of their normal forms and their SSS, ');
	writeLn('in increasing order with respect to the D-ordering; "D" stands for "Delta".');
	writeLn;
	for p:= 1 to braidList.L do
		begin
			write(p, ': ');
			w:= braidList.V[p];
			write(paddedAltNormal(w));
			u:= braidList.V[mu3SSS[p]];
			write('; mu3SSS = ');
			write(paddedAltNormal(u));
			write(' SSS: (', altDecode3(SSSList[p]), ')');
			writeLn;
		end;
end;		{ buildSSS3 }

procedure displayMu3;
var
	p: integer;
	u, v, w, uu, vv: string;
begin
	writeLn('Values of mu3 for the ', braidList.L, ' positive braids of B_3 of length at most ', lengthBound, ':');
	writeLn('enumerated in increasing order; no value means "itself".');
	writeLn;
	writeFix('Codes of altern.form', 27);
	writeFix('Alternating form', 2*lengthBound+13);
	writeFix('Garside form', 2*lengthBound+15);
	writeLn;
	for p:= 1 to braidList.L do
		begin
			w:= braidList.V[p];
			v:= braidList.V[mu3[p]];
			u:= braidList.V[mu3Conj[p]];
			vv:= braidList.V[mu3SSS[p]];
			uu:= braidList.V[mu3SSSConj[p]];
			if p <= 9 then
				write(p, '   :')
			else if p <= 99 then
				write(p, '  :')
			else if p <= 999 then
				write(p, ' :')
			else write(p, ':');
			write(paddedAltCode3(w), ' -> ');
			write(paddedAltCode3(v), ' / ', paddedAltCode3(vv));
			write(' by ');
			write(paddedAltCode3(u), ' / ', paddedAltCode3(uu));
			write('   ');
			write(paddedAltNormal(w), ' -> ');
			write(paddedAltNormal(v), ' by ');
			write(paddedAltNormal(u), '   ');
			write('   ');
			write(paddedGarNormal(w), ' -> ');
			write(paddedGarNormal(v), ' by ');
			write(paddedGarNormal(u), '   ');
			writeLn;
		end;
end;		{ displayMu3 }

procedure displayMu3SSS;
var
	p: integer;
	u, v, w: string;
begin
	writeLn('Values of mu3SSS for the ', braidList.L, ' positive braids of B_3 of length at most ', lengthBound, ':');
	writeLn('enumerated in increasing order; no value means "itself".');
	writeLn;
	writeFix('Codes of altern.form', 27);
	writeFix('Alternating form', 2*lengthBound+13);
	writeFix('Garside form', 2*lengthBound+15);
	writeLn;
	for p:= 1 to braidList.L do
		begin
			w:= braidList.V[p];
			v:= braidList.V[mu3SSS[p]];
			u:= braidList.V[mu3SSSConj[p]];
			if p <= 9 then
				write(p, '   :')
			else if p <= 99 then
				write(p, '  :')
			else if p <= 999 then
				write(p, ' :')
			else write(p, ':');
			write(paddedAltCode3(w), ' -> ');
			if mu3[p] < p then
				write(paddedAltCode3(v), ' by ', paddedAltCode3(u))
			else
				write(paddedAltCode3(''), ' by ', paddedAltCode3(''));
			write('   ');
			write(paddedAltNormal(w), ' -> ');
			write(paddedAltNormal(v), ' by ');
			write(paddedAltNormal(u), '   ');
			write('   ');
			write(paddedGarNormal(w), ' -> ');
			write(paddedGarNormal(v), ' by ');
			write(paddedGarNormal(u), '   ');
			writeLn;
		end;
end;		{ displayMu3SSS }

procedure studyMu3;
var
	r, p: integer;
	w, u, v: string;
	B: braid;
begin
	writeLn('One compares the values of mu(w) and mu(uwv) for some fixed u, v.');
	write('Enter the factor "u": ');
	readLn(u);
	write('Enter the factor "v": ');
	readLn(v);
	for r:= 1 to braidList.L do
		begin
			w:= braidList.V[r];
			BDef(B, concat(u, w, v));
			p:= rank(B);
			if p > 0 then
				begin
					writeFix(altCode3(w), 9);
					write(' -> ');
					writeFix(altCode3(braidList.V[mu3[r]]), 9);
					write(' by ');
					writeFix(altCode3(braidList.V[mu3Conj[r]]), 9);
					write(' ; ');
					writeFix(altCode3(braidList.V[p]), 9);
					write(' --> ');
					writeFix(altCode3(braidList.V[mu3[p]]), 9);
					write(' by ');
					writeFix(altCode3(braidList.V[mu3Conj[p]]), 9);
					writeLn;
				end;
		end;
end;		{ studyMu3 }

procedure experiment;
begin
end;		{ experiment }


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Control
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure initialize;
var log: char;
begin
	writeLn();
	hRule;
  writeLn('   Braids, version 1.1 (Sep.10, 2016)');
  writeLn('   Patrick DEHORNOY, LMNO, Universite de Caen, 14032 Caen, France');
	writeLn('   patrick.dehornoy@unicaen.fr');
	writeLn('   www.math.unicaen.fr/~dehornoy/');
	hRule;
  writeLn;
	BOrderOption := 'D';
	writeLn('The current bounds for the braids are ', BMaxLength : 0, ' crossings on ', BMaxWidth : 0, ' strands. ');
	writeLn('The default braid order is the original Dehornoy ordering (sigma_1 large). ');
end;	// initialize

procedure help;
begin
	hRule;
  writeLn('Explanations about this program');
  writeLn;
	writeLn('This program performs a few braid operations, in particular involving orders.');
	writeLn('The main comparison procedure is handle reduction, which, starting with ');
	writeLn('an arbitrary braid word, returns an equivalent sigma-definite word.');
	writeLn;
  writeLn('Reference:');
  writeLn(' [1] A fast method for comparing braids, Adv. in Math. 268 (2003) 156-197');
  writeLn(' [2] (with I.Dynnikov, D.Rolfsen, B.Wiest) Ordering Braids');
  writeLn('        Mathematical Surveys and Monographs vol. 148, Amer. Math. Soc. (2008)');
	hRule;
end;	// help

procedure chooseOrdering;
begin
	writeLn('  Choose a braid ordering:');
	writeLn('    Original Dehornoy ordering (sigma_1 large) .......................... D');
	writeLn('    Flipped Dehornoy ordering (sigma_n large) ........................... F');
	writeLn('    Dubrovina-Dubrovin-ordering ......................................... U');
	write('> ');
	readLn(BOrderOption);
	writeLn('From now on, the ordering is ', BOrderName, '.');
end;	// chooseOrdering

procedure quit;
begin
end;

procedure askMenu;
begin
	writeLn;
  hRule;
  writeLn('MAIN MENU Choose a procedure:');
  writeLn('  Help');
	writeLn('    Explanations about the program and references ....................... h ');
	writeLn('    Change the reference braid ordering ................................. O');
  writeLn('  Garside structure');
	writeLn('    Display the symmetric normal form ................................... N');
  writeLn('  Ordered structure');
	writeLn('    Display the alternating form ........................................ A');
	writeLn('    Reduce the handles .................................................. H');
	writeLn('    Compare two braids .................................................. C');
	//writeLn('  Experiment ................................................. X');
	//writeLn('CONJUGACY IN B3+');
	//writeLn('  Build complete list of SSSs (up to length ', maxLengthInBraidList, ') .............. B');
	//writeLn('  Display mu3 ................................................ M');
	//writeLn('  Display mu3SSS ............................................. S');
	//writeLn('  Investigate mu3  ........................................... I');
	writeLn('  Quit .................................................................. Q');
	write('    > ');
	readLn(menu);
	writeLn;
end;

begin
	initialize;
	repeat
		askMenu;
		case menu of
			'A': writeAlt;
			'B': buildSSS3;
			'C': compare;
      'h': help;
			'H': reduceHandles;
			'M': displayMu3;
			'S': displayMu3SSS;
			'N': symmetric;
			'O': chooseOrdering;
			'I': studyMu3;
			'X': experiment;
		end;
	until menu = 'Q';
	quit;
end.
